import { FormItem, Col } from 'element-ui'
import { FormComponents } from './config'

import {
  getValue,
  setValue,
  copyDeep,
  effectiveValue,
  isHidden
} from '../../utils'
import './style.css'

import {
  handlerTips
} from './components'

/**
 * 生成表单项
 * 区别表单控件的 type、props属性 在此处理
 * */
export function createItem (that, item) {
  const h = that.$createElement
  const fm = that
  const {
    type = '', // 表单组件类型，具体参考 ./config.js
    filed = '', // 字段名称，用于初始化form对象
    style = {}, // 接受一个字符串、对象，或对象组成的数组
    classList = {}, // 与 `v-bind:class` 的 API 相同，接受一个字符串、对象或字符串和对象组成的数组
    domProps = {}, // DOM property
    nativeOn = {}, // // 仅用于组件，用于监听原生事件，而不是组件内部使用，`vm.$emit` 触发的事件。
    directives = [], // 自定义指令。注意，你无法对 `binding` 中的 `oldValue`，赋值，因为 Vue 已经自动为你进行了同步。
    scopedSlots = {}, // 作用域插槽的格式为{ name: props => VNode | Array<VNode> }
    slot = '', // 如果组件是其它组件的子组件，需为插槽指定名称
    key = '', // 其它特殊顶层 property
    ref = '', // 其它特殊顶层 property
    refInFor = true, // 如果你在渲染函数中给多个元素都应用了相同的 ref 名，那么 `$refs.myRef` 会变成一个数组。
    render, // 自定义，优先级顺序 1
    formItemRender, // 在formItem中自定义，优先级顺序 2
    tips = {}, // 注释
    component // 组件
  } = item

  if (typeof render === 'function') {
    return render(h, fm.form)
  }

  if (typeof formItemRender === 'function') {
    return handlerFormItem(item, formItemRender(h, { form: fm.form, searchConfig: fm.searchConfig }), fm)
  }
  const renderOptions = handlerComponentConfig(fm, item)

  if (component) {
    const VN = h(component, renderOptions)
    return handlerFormItem(item, VN, fm)
  }

  if (type === 'text') {
    return handlerFormItem(item, h('span', {
      style,
      class: classList
    }, getValue(fm.form, filed)), fm)
  }
  if (!FormComponents[type]) {
    console.error('There is no matching type', item)
    return
  }

  if (typeof FormComponents[type] == 'function') {
    const VN = FormComponents[type].call(fm, h, item, renderOptions)
    return handlerFormItem(item, VN, fm)
  }

  return handlerFormItem(item, h(FormComponents[type], renderOptions, null), fm)
}

export function handlerComponentConfig (that, item) {
  const { filed, componentProps = {} } = item
  const {
    props = {},
    attrs = {},
    class: comClass = {},
    style = {},
    on = {}
  } = componentProps

  return {
    ...componentProps,
    props: {
      clearable: true,
      size: that.size,
      ...props,
      value: getValue(that.form, filed),
      isEdit: that.isEdit,
    },
    attrs: {
      ...attrs,
      onChange: (val) => {
        setValue(that.form, filed, val)
        attrs.onChange && attrs.onChange(val)
      }
    },
    class: {
      ...comClass
    },
    style: {
      ...that.allControlStyle,
      ...style
    },
    on: {
      ...on,
      input: (val) => {
        setValue(that.form, filed, val)
        on.input && on.input(val)
      }
    }
  }
}

/**
 * 处理表单项
 * @param item 虚拟DOM options
 * @param VNode 子级虚拟DOM
 * @param fm form组件的this
 * */
export function handlerFormItem (item = {}, VNode, fm = {}) {
  const h = fm.$createElement
  const { allFormItemStyle } = fm
  const {
    label = '',
    formItemProps,
    filed = '',
    span,
    tips,
    formItemStyle = {},
    controlStyle = {}
  } = item
  let childrenVNode = VNode
  if (!Array.isArray(VNode)) {
    childrenVNode = [childrenVNode]
  }

  if (tips) {
    childrenVNode.push(handlerTips(h, item, fm))
  }

  const formItem = h(FormItem,
    {
      props: {
        ...formItemProps,
        prop: filed,
        label
      }
    },
    [
      h('div', {
        style: {
          // ...fm.allControlStyle,
          ...controlStyle
        }
      }, childrenVNode)
    ]
  )

  return h(Col, {
    props: {
      span
    },
    style: {
      ...allFormItemStyle,
      ...formItemStyle
    }
  }, [formItem])
}

/**
 * 初始化表单结构
 * @param form value
 * @param config config
 * @param emptyForm 空对象
 * 该方法功能，value和config的数据相结合
 * */
export function initForm (form, config, emptyForm) {
  const newForm = copyDeep(form)
  formAndConfigCombination(newForm, config, emptyForm)
  return newForm
}

function formAndConfigCombination (form, children, emptyForm) {
  children.forEach((item) => {
    const defaultVal = item.props && item.props.value
    if (typeof item.filed !== 'undefined') {
      const propsValue = getValue(form, item.filed)
      const value = effectiveValue(propsValue) ? propsValue : defaultVal || null
      setValue(form, item.filed, value)
      setValue(emptyForm, item.filed, null)
    }
    if (Array.isArray(item.children) && item.children.length) {
      formAndConfigCombination(form, item.children, emptyForm)
    }
  })
}

/**
 * 根据传入的数据，填充入表单中
 * @param that form this
 * @param form 组件里的 form 对象
 * @param data v-model传入的数据
 * */
export function setFormData (that, form, data) {
  if (Object.keys(data).length === 0) {
    that.clear()
    return
  }
  const dataKeys = []
  for (let key in data) {
    dataKeys.push(key)
    form[key] = data[key]
  }
  for (let key in form) {
    if (!dataKeys.includes(key)) {
      form[key] = null
    }
  }
}

/**
 * 生成表单子项
 * @param that form对象的this
 * @param item config 子元素配置数据
 * */
export function createItemChildren (that, item) {
  const fm = that
  const { children } = item
  const items = []

  for (let i = 0; i < children.length; i++) {
    const childrenItem = children[i]
    const { hidden } = childrenItem
    if (isHidden(hidden, fm.form, fm.config)) {
      continue
    }
    if (childrenItem.children && childrenItem.children.length) {
      items.push(createItemChildren(fm, childrenItem))
    } else {
      items.push(createItem(fm, childrenItem))
    }
  }

  return handlerFormItem(item, items, fm)
}
