import { get, set, cloneDeep, isEqual } from 'lodash'
import moment from 'moment'

export function valueEqual (obj, obj2) {
  return isEqual(obj, obj2)
}
export function getValue (obj, key) {
  return get(obj, key)
}
export function setValue (obj, key, value) {
  return set(obj, key, value)
}
export function copyDeep (value) {
  return cloneDeep(value)
}

export function handlerMoment (value) {
  return moment(value)
}

export function isVNode (value) {
  return Object.getPrototypeOf(value).constructor.name === 'VNode'
}

export function isObject (value) {
  return Object.prototype.toString.call(value) === '[object Object]'
}
export function isArray (value) {
  return Object.prototype.toString.call(value) === '[object Array]'
}
export function isDate (value) {
  return Object.prototype.toString.call(value) === '[object Date]'
}
export function isNaN (value) {
  return Object.prototype.toString.call(value) === '[object Number]' && value !== value
}
export function isNumber (value) {
  return typeof value === 'number'
}
export function isString (value) {
  return typeof value === 'string'
}
export function isFunction (value) {
  return typeof value === 'function'
}
export function isSymbol (value) {
  return typeof value === 'symbol'
}
export function isUndefined (value) {
  return typeof value === 'undefined'
}
export function isBoolean (value) {
  return typeof value === 'boolean'
}
export function isNull (value) {
  return Object.prototype.toString.call(value) === '[object Null]'
}

/**
 * 该函数会判断传入的参数是否是一个有效值，这个值有效判断允许为 number 0 和 null
 * @param {any} value
 * */
export function effectiveValue (value) {
  return (typeof value !== 'undefined' && value !== '' && !isNaN(value))
}

/**
 * 设置值为空
 * */
export function setValueEmpty (value) {
  let newValue = ''

  if (isObject(value)) {
    const objArr = Object.keys(value)
    newValue = {}
    if (objArr.length > 0) {
      objArr.forEach((item) => {
        if (isObject(item) || isArray(item)) {
          newValue[item] = setValueEmpty(value[item])
        } else {
          newValue[item] = ''
        }
      })
    }
    return newValue
  }
  if (isArray(value)) {
    newValue = []
    if (value.length) {
      let hasObj = 0
      for (let i = 0; i < value.length; i++) {
        const item = value[i]
        if (isObject(item) || isArray(item)) {
          newValue.push(setValueEmpty(item))
          hasObj = 1
        }
      }
      if (hasObj === 0) {
        newValue = []
      }
    }
    return newValue
  }
  return newValue
}

/**
 * 防抖
 * 一个监听事件，在规定的时间间隔执行
 * 例如 window.addEventListener('resize', function(e) {})，
 * 改为 1秒执行一次，window.addEventListener('resize', debounce((e) => {
 *   console.log(e)
 * }, 1000))
 * @param fn 回调函数
 * @param delay 延迟时间 ms
 * */
export function debounce (fn, delay) {
  let timer = null
  delay = typeof delay === 'number' ? delay : 500
  return function () {
    if (timer) {
      clearTimeout(timer)
    }
    timer = setTimeout(() => {
      fn.apply(this, arguments)
      timer = null
    }, delay)
  }
}

/**
 * 把value处理成千分位
 * @param value is Number
 * @param precision 最多支持小数点后3位
 * */
export function handlerThousands (value, precision) {
  let newValue = Number(value)
  if (isNaN(newValue)) {
    console.error('The argument should be a valid numeric type.')
    return null
  }

  newValue = value.toString()

  if (newValue.indexOf('.') !== -1) {
    const values = newValue.split('.')
    const initNum = values[0]
    let decimal = values[1]
    let coverStr = '' // 补位
    if (typeof precision === 'number' && precision > 0) {
      // 如果指定的小数位大于实际小数位数，则补位
      if (precision > decimal.length) {
        for (let i = 0; i < precision - decimal.length; i++) {
          coverStr += '0'
        }
      }
      decimal = decimal.substr(0, precision)
      return regThousands(initNum) + '.' + decimal + coverStr
    }
    return regThousands(initNum)
  } else {
    return regThousands(newValue)
  }
}

export function regThousands (value) {
  return value.replace(/\B(?=(\d{3})+(?!\d))/g, ",")
}

export function isHidden(value, data, config) {
  if (isFunction(value) && value(data, config)) {
    return true
  }
  if (isBoolean(value) && value) {
    return true
  }
  return false
}

/**
 * 获取时间
 * */
export function getDateTime (value) {
  value = value || new Date()
  const dateInstance = new Date(value)
  const today = dateInstance.getDay()
  const year = dateInstance.getFullYear()
  const month = dateInstance.getMonth() + 1
  const date = dateInstance.getDate()
  const hours = dateInstance.getHours()
  const minutes = dateInstance.getMinutes()
  const seconds = dateInstance.getSeconds()
  const milliseconds = dateInstance.getMilliseconds()
  return {
    dateInstance,
    today,
    year,
    month,
    date,
    hours,
    minutes,
    seconds,
    milliseconds
  }
}
